@@ -1,10 +1,10 @@
-/* $Id: strformatrt.cpp 115341 2017-05-10 14:00:11Z fmehnert $ */
+/* $Id: strformatrt.cpp 133080 2019-09-03 12:51:00Z bird $ */
 /** @file
  * IPRT - IPRT String Formatter Extensions.
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -41,8 +41,9 @@
 #include <iprt/string.h>
 #include <iprt/stdarg.h>
 #ifdef IN_RING3
+# include <iprt/errcore.h>
 # include <iprt/thread.h>
-# include <iprt/err.h>
+# include <iprt/utf16.h>
 #endif
 #include <iprt/ctype.h>
 #include <iprt/time.h>
@@ -60,6 +61,9 @@
 *   Global Variables                                                                                                             *
 *********************************************************************************************************************************/
 static char g_szHexDigits[17] = "0123456789abcdef";
+#ifdef IN_RING3
+static char g_szHexDigitsUpper[17] = "0123456789ABCDEF";
+#endif
 
 
 /**
@@ -82,9 +86,9 @@ static size_t rtstrFormatIPv6HexWord(char *pszDst, uint16_t uWord)
     off = 0;
     switch (cDigits)
     {
-        case 4: pszDst[off++] = g_szHexDigits[(uWord >> 12) & 0xf]; /* fall thru */
-        case 3: pszDst[off++] = g_szHexDigits[(uWord >>  8) & 0xf]; /* fall thru */
-        case 2: pszDst[off++] = g_szHexDigits[(uWord >>  4) & 0xf]; /* fall thru */
+        case 4: pszDst[off++] = g_szHexDigits[(uWord >> 12) & 0xf]; RT_FALL_THRU();
+        case 3: pszDst[off++] = g_szHexDigits[(uWord >>  8) & 0xf]; RT_FALL_THRU();
+        case 2: pszDst[off++] = g_szHexDigits[(uWord >>  4) & 0xf]; RT_FALL_THRU();
         case 1: pszDst[off++] = g_szHexDigits[(uWord >>  0) & 0xf];
             break;
     }
@@ -669,7 +673,7 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
             /* Group 3 */
 
             /*
-             * Base name printing.
+             * Base name printing, big endian UTF-16.
              */
             case 'b':
             {
@@ -699,6 +703,56 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
                         return pfnOutput(pvArgOutput, pszLastSep, psz - pszLastSep);
                     }
 
+                    /* %lRbs */
+                    case 's':
+                        if (chArgSize == 'l')
+                        {
+                            /* utf-16BE -> utf-8 */
+                            int         cchStr;
+                            PCRTUTF16   pwszStr = va_arg(*pArgs, PRTUTF16);
+
+                            if (RT_VALID_PTR(pwszStr))
+                            {
+                                cchStr = 0;
+                                while (cchStr < cchPrecision && pwszStr[cchStr] != '\0')
+                                    cchStr++;
+                            }
+                            else
+                            {
+                                static RTUTF16  s_wszBigNull[] =
+                                {
+                                    RT_H2BE_U16_C((uint16_t)'<'), RT_H2BE_U16_C((uint16_t)'N'), RT_H2BE_U16_C((uint16_t)'U'),
+                                    RT_H2BE_U16_C((uint16_t)'L'), RT_H2BE_U16_C((uint16_t)'L'), RT_H2BE_U16_C((uint16_t)'>'), '\0'
+                                };
+                                pwszStr = s_wszBigNull;
+                                cchStr  = RT_ELEMENTS(s_wszBigNull) - 1;
+                            }
+
+                            cch = 0;
+                            if (!(fFlags & RTSTR_F_LEFT))
+                                while (--cchWidth >= cchStr)
+                                    cch += pfnOutput(pvArgOutput, " ", 1);
+                            cchWidth -= cchStr;
+                            while (cchStr-- > 0)
+                            {
+/** @todo \#ifndef IN_RC*/
+#ifdef IN_RING3
+                                RTUNICP Cp = 0;
+                                RTUtf16BigGetCpEx(&pwszStr, &Cp);
+                                char *pszEnd = RTStrPutCp(szBuf, Cp);
+                                *pszEnd = '\0';
+                                cch += pfnOutput(pvArgOutput, szBuf, pszEnd - szBuf);
+#else
+                                szBuf[0] = (char)(*pwszStr++ >> 8);
+                                cch += pfnOutput(pvArgOutput, szBuf, 1);
+#endif
+                            }
+                            while (--cchWidth >= 0)
+                                cch += pfnOutput(pvArgOutput, " ", 1);
+                            return cch;
+                        }
+                    RT_FALL_THRU();
+
                     default:
                         AssertMsgFailed(("Invalid status code format type '%.10s'!\n", pszFormatOrg));
                         break;
@@ -766,20 +820,37 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
 
 
             /*
-             * hex dumping and COM/XPCOM.
+             * hex dumping, COM/XPCOM, human readable sizes.
              */
             case 'h':
             {
-                switch (*(*ppszFormat)++)
+                ch = *(*ppszFormat)++;
+                switch (ch)
                 {
                     /*
                      * Hex stuff.
                      */
                     case 'x':
+                    case 'X':
                     {
                         uint8_t *pu8 = va_arg(*pArgs, uint8_t *);
+                        uint64_t uMemAddr;
+                        int      cchMemAddrWidth;
+
                         if (cchPrecision < 0)
                             cchPrecision = 16;
+
+                        if (ch == 'x')
+                        {
+                            uMemAddr = (uintptr_t)pu8;
+                            cchMemAddrWidth = sizeof(pu8) * 2;
+                        }
+                        else
+                        {
+                            uMemAddr = va_arg(*pArgs, uint64_t);
+                            cchMemAddrWidth = uMemAddr > UINT32_MAX || uMemAddr + cchPrecision > UINT32_MAX ? 16 : 8;
+                        }
+
                         if (pu8)
                         {
                             switch (*(*ppszFormat)++)
@@ -798,10 +869,12 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
                                     while (off < cchPrecision)
                                     {
                                         int i;
-                                        cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "%s%0*p %04x:", off ? "\n" : "", sizeof(pu8) * 2, (uintptr_t)pu8, off);
+                                        cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "%s%0*llx/%04x:",
+                                                           off ? "\n" : "", cchMemAddrWidth, uMemAddr + off, off);
                                         for (i = 0; i < cchWidth && off + i < cchPrecision ; i++)
                                             cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
-                                                               off + i < cchPrecision ? !(i & 7) && i ? "-%02x" : " %02x" : "   ", pu8[i]);
+                                                               off + i < cchPrecision ? !(i & 7) && i ? "-%02x" : " %02x" : "   ",
+                                                               pu8[i]);
                                         while (i++ < cchWidth)
                                             cch += pfnOutput(pvArgOutput, "   ", 3);
 
@@ -820,6 +893,65 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
                                     return cch;
                                 }
 
+                                /*
+                                 * Regular hex dump with dittoing.
+                                 */
+                                case 'D':
+                                {
+                                    int offEndDupCheck;
+                                    int cDuplicates = 0;
+                                    int off = 0;
+                                    cch = 0;
+
+                                    if (cchWidth <= 0)
+                                        cchWidth = 16;
+                                    offEndDupCheck = cchPrecision - cchWidth;
+
+                                    while (off < cchPrecision)
+                                    {
+                                        int i;
+                                        if (   off >= offEndDupCheck
+                                            || off <= 0
+                                            || memcmp(pu8, pu8 - cchWidth, cchWidth) != 0
+                                            || (   cDuplicates == 0
+                                                && (   off + cchWidth >= offEndDupCheck
+                                                    || memcmp(pu8 + cchWidth, pu8, cchWidth) != 0)) )
+                                        {
+                                            if (cDuplicates > 0)
+                                            {
+                                                cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "\n%.*s ****  <ditto x %u>",
+                                                                   cchMemAddrWidth, "****************", cDuplicates);
+                                                cDuplicates = 0;
+                                            }
+
+                                            cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "%s%0*llx/%04x:",
+                                                               off ? "\n" : "", cchMemAddrWidth, uMemAddr + off, off);
+                                            for (i = 0; i < cchWidth && off + i < cchPrecision ; i++)
+                                                cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                                     off + i < cchPrecision ? !(i & 7) && i
+                                                                   ? "-%02x" : " %02x" : "   ",
+                                                                   pu8[i]);
+                                            while (i++ < cchWidth)
+                                                cch += pfnOutput(pvArgOutput, "   ", 3);
+
+                                            cch += pfnOutput(pvArgOutput, " ", 1);
+
+                                            for (i = 0; i < cchWidth && off + i < cchPrecision; i++)
+                                            {
+                                                uint8_t u8 = pu8[i];
+                                                cch += pfnOutput(pvArgOutput, u8 < 127 && u8 >= 32 ? (const char *)&u8 : ".", 1);
+                                            }
+                                        }
+                                        else
+                                            cDuplicates++;
+
+                                        /* next */
+                                        pu8 += cchWidth;
+                                        off += cchWidth;
+                                    }
+                                    return cch;
+                                }
+
                                 /*
                                  * Hex string.
                                  */
@@ -827,7 +959,11 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
                                 {
                                     if (cchPrecision-- > 0)
                                     {
-                                        cch = RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "%02x", *pu8++);
+                                        if (ch == 'x')
+                                            cch = RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "%02x", *pu8++);
+                                        else
+                                            cch = RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "%0*llx: %02x",
+                                                              cchMemAddrWidth, uMemAddr, *pu8++);
                                         for (; cchPrecision > 0; cchPrecision--, pu8++)
                                             cch += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, " %02x", *pu8);
                                         return cch;
@@ -871,6 +1007,127 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
                     }
 #endif /* IN_RING3 */
 
+                    /*
+                     * Human readable sizes.
+                     */
+                    case 'c':
+                    case 'u':
+                    {
+                        unsigned    i;
+                        ssize_t     cchBuf;
+                        uint64_t    uValue;
+                        uint64_t    uFraction = 0;
+                        const char *pszPrefix = NULL;
+                        char        ch2 = *(*ppszFormat)++;
+                        AssertMsgReturn(ch2 == 'b' || ch2 == 'B' || ch2 == 'i', ("invalid type '%.10s'!\n", pszFormatOrg), 0);
+                        uValue = va_arg(*pArgs, uint64_t);
+
+                        if (!(fFlags & RTSTR_F_PRECISION))
+                            cchPrecision = 1; /** @todo default to flexible decimal point. */
+                        else if (cchPrecision > 3)
+                            cchPrecision = 3;
+                        else if (cchPrecision < 0)
+                            cchPrecision = 0;
+
+                        if (ch2 == 'b' || ch2 == 'B')
+                        {
+                            static const struct
+                            {
+                                const char *pszPrefix;
+                                uint8_t     cShift;
+                                uint64_t    cbMin;
+                                uint64_t    cbMinZeroPrecision;
+                            } s_aUnits[] =
+                            {
+                                {  "Ei", 60, _1E,  _1E*2 },
+                                {  "Pi", 50, _1P,  _1P*2 },
+                                {  "Ti", 40, _1T,  _1T*2 },
+                                {  "Gi", 30, _1G,  _1G64*2 },
+                                {  "Mi", 20, _1M,  _1M*2 },
+                                {  "Ki", 10, _1K,  _1K*2 },
+                            };
+                            for (i = 0; i < RT_ELEMENTS(s_aUnits); i++)
+                                if (   uValue >= s_aUnits[i].cbMin
+                                    && (cchPrecision > 0 || uValue >= s_aUnits[i].cbMinZeroPrecision))
+                                {
+                                    if (cchPrecision != 0)
+                                    {
+                                        uFraction   = uValue & (RT_BIT_64(s_aUnits[i].cShift) - 1);
+                                        uFraction  *= cchPrecision == 1 ? 10 : cchPrecision == 2 ? 100 : 1000;
+                                        uFraction >>= s_aUnits[i].cShift;
+                                    }
+                                    uValue  >>= s_aUnits[i].cShift;
+                                    pszPrefix = s_aUnits[i].pszPrefix;
+                                    break;
+                                }
+                        }
+                        else
+                        {
+                            static const struct
+                            {
+                                const char *pszPrefix;
+                                uint64_t    cbFactor;
+                                uint64_t    cbMinZeroPrecision;
+                            } s_aUnits[] =
+                            {
+                                {  "E", UINT64_C(1000000000000000000), UINT64_C(1010000000000000000),  },
+                                {  "P", UINT64_C(1000000000000000),    UINT64_C(1010000000000000),     },
+                                {  "T", UINT64_C(1000000000000),       UINT64_C(1010000000000),        },
+                                {  "G", UINT64_C(1000000000),          UINT64_C(1010000000),           },
+                                {  "M", UINT64_C(1000000),             UINT64_C(1010000),              },
+                                {  "k", UINT64_C(1000),                UINT64_C(1010),                 },
+                            };
+                            for (i = 0; i < RT_ELEMENTS(s_aUnits); i++)
+                                if (   uValue >= s_aUnits[i].cbFactor
+                                    && (cchPrecision > 0 || uValue >= s_aUnits[i].cbMinZeroPrecision))
+                                {
+                                    if (cchPrecision == 0)
+                                        uValue /= s_aUnits[i].cbFactor;
+                                    else
+                                    {
+                                        uFraction   = uValue % s_aUnits[i].cbFactor;
+                                        uValue      = uValue / s_aUnits[i].cbFactor;
+                                        uFraction  *= cchPrecision == 1 ? 10 : cchPrecision == 2 ? 100 : 1000;
+                                        uFraction  += s_aUnits[i].cbFactor >> 1;
+                                        uFraction  /= s_aUnits[i].cbFactor;
+                                    }
+                                    pszPrefix = s_aUnits[i].pszPrefix;
+                                    break;
+                                }
+                        }
+
+                        cchBuf = RTStrFormatU64(szBuf, sizeof(szBuf), uValue, 10, 0, 0, 0);
+                        if (pszPrefix)
+                        {
+                            if (cchPrecision)
+                            {
+                                szBuf[cchBuf++] = '.';
+                                cchBuf += RTStrFormatU64(&szBuf[cchBuf], sizeof(szBuf) - cchBuf, uFraction, 10, cchPrecision, 0,
+                                                         RTSTR_F_ZEROPAD | RTSTR_F_WIDTH);
+                            }
+                            if (fFlags & RTSTR_F_BLANK)
+                                szBuf[cchBuf++] = ' ';
+                            szBuf[cchBuf++] = *pszPrefix++;
+                            if (*pszPrefix && ch2 != 'B')
+                                szBuf[cchBuf++] = *pszPrefix;
+                        }
+                        else if (fFlags & RTSTR_F_BLANK)
+                            szBuf[cchBuf++] = ' ';
+                        if (ch == 'c')
+                            szBuf[cchBuf++] = 'B';
+                        szBuf[cchBuf] = '\0';
+
+                        cch = 0;
+                        if ((fFlags & RTSTR_F_WIDTH) && !(fFlags & RTSTR_F_LEFT))
+                            while (cchBuf < cchWidth)
+                            {
+                                cch += pfnOutput(pvArgOutput, fFlags & RTSTR_F_ZEROPAD ? "0" : " ", 1);
+                                cchWidth--;
+                            }
+                        cch += pfnOutput(pvArgOutput, szBuf, cchBuf);
+                        return cch;
+                    }
+
                     default:
                         AssertMsgFailed(("Invalid status code format type '%.10s'!\n", pszFormatOrg));
                         return 0;
@@ -1055,80 +1312,226 @@ DECLHIDDEN(size_t) rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, co
             }
 
 #ifdef IN_RING3
+
             /*
-             * Group 5, XML / HTML escapers.
+             * Group 5, XML / HTML, JSON and URI escapers.
              */
             case 'M':
             {
                 char chWhat = (*ppszFormat)[0];
-                bool fAttr  = chWhat == 'a';
-                char chType = (*ppszFormat)[1];
-                AssertMsgBreak(chWhat == 'a' || chWhat == 'e', ("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
-                *ppszFormat += 2;
-                switch (chType)
+                if (chWhat == 'a' || chWhat == 'e')
                 {
-                    case 's':
+                    /* XML attributes and element values. */
+                    bool fAttr  = chWhat == 'a';
+                    char chType = (*ppszFormat)[1];
+                    *ppszFormat += 2;
+                    switch (chType)
                     {
-                        static const char   s_szElemEscape[] = "<>&\"'";
-                        static const char   s_szAttrEscape[] = "<>&\"\n\r"; /* more? */
-                        const char * const  pszEscape =  fAttr ?             s_szAttrEscape  :             s_szElemEscape;
-                        size_t       const  cchEscape = (fAttr ? RT_ELEMENTS(s_szAttrEscape) : RT_ELEMENTS(s_szElemEscape)) - 1;
-                        size_t      cchOutput = 0;
-                        const char *pszStr    = va_arg(*pArgs, char *);
-                        ssize_t     cchStr;
-                        ssize_t     offCur;
-                        ssize_t     offLast;
+                        case 's':
+                        {
+                            static const char   s_szElemEscape[] = "<>&\"'";
+                            static const char   s_szAttrEscape[] = "<>&\"\n\r"; /* more? */
+                            const char * const  pszEscape =  fAttr ?             s_szAttrEscape  :             s_szElemEscape;
+                            size_t       const  cchEscape = (fAttr ? RT_ELEMENTS(s_szAttrEscape) : RT_ELEMENTS(s_szElemEscape)) - 1;
+                            size_t      cchOutput = 0;
+                            const char *pszStr    = va_arg(*pArgs, char *);
+                            ssize_t     cchStr;
+                            ssize_t     offCur;
+                            ssize_t     offLast;
 
-                        if (!VALID_PTR(pszStr))
-                            pszStr = "<NULL>";
-                        cchStr = RTStrNLen(pszStr, (unsigned)cchPrecision);
+                            if (!VALID_PTR(pszStr))
+                                pszStr = "<NULL>";
+                            cchStr = RTStrNLen(pszStr, (unsigned)cchPrecision);
+
+                            if (fAttr)
+                                cchOutput += pfnOutput(pvArgOutput, "\"", 1);
+                            if (!(fFlags & RTSTR_F_LEFT))
+                                while (--cchWidth >= cchStr)
+                                    cchOutput += pfnOutput(pvArgOutput, " ", 1);
+
+                            offLast = offCur = 0;
+                            while (offCur < cchStr)
+                            {
+                                if (memchr(pszEscape, pszStr[offCur], cchEscape))
+                                {
+                                    if (offLast < offCur)
+                                        cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+                                    switch (pszStr[offCur])
+                                    {
+                                        case '<':   cchOutput += pfnOutput(pvArgOutput, "&lt;", 4); break;
+                                        case '>':   cchOutput += pfnOutput(pvArgOutput, "&gt;", 4); break;
+                                        case '&':   cchOutput += pfnOutput(pvArgOutput, "&amp;", 5); break;
+                                        case '\'':  cchOutput += pfnOutput(pvArgOutput, "&apos;", 6); break;
+                                        case '"':   cchOutput += pfnOutput(pvArgOutput, "&quot;", 6); break;
+                                        case '\n':  cchOutput += pfnOutput(pvArgOutput, "&#xA;", 5); break;
+                                        case '\r':  cchOutput += pfnOutput(pvArgOutput, "&#xD;", 5); break;
+                                        default:
+                                            AssertFailed();
+                                    }
+                                    offLast = offCur + 1;
+                                }
+                                offCur++;
+                            }
+                            if (offLast < offCur)
+                                cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
 
-                        if (fAttr)
-                            cchOutput += pfnOutput(pvArgOutput, "\"", 1);
-                        if (!(fFlags & RTSTR_F_LEFT))
                             while (--cchWidth >= cchStr)
                                 cchOutput += pfnOutput(pvArgOutput, " ", 1);
-
-                        offLast = offCur = 0;
-                        while (offCur < cchStr)
-                        {
-                            if (memchr(pszEscape, pszStr[offCur], cchEscape))
-                            {
-                                if (offLast < offCur)
-                                    cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
-                                switch (pszStr[offCur])
-                                {
-                                    case '<':   cchOutput += pfnOutput(pvArgOutput, "&lt;", 4); break;
-                                    case '>':   cchOutput += pfnOutput(pvArgOutput, "&gt;", 4); break;
-                                    case '&':   cchOutput += pfnOutput(pvArgOutput, "&amp;", 5); break;
-                                    case '\'':  cchOutput += pfnOutput(pvArgOutput, "&apos;", 6); break;
-                                    case '"':   cchOutput += pfnOutput(pvArgOutput, "&quot;", 6); break;
-                                    case '\n':  cchOutput += pfnOutput(pvArgOutput, "&#xA;", 5); break;
-                                    case '\r':  cchOutput += pfnOutput(pvArgOutput, "&#xD;", 5); break;
-                                    default:
-                                        AssertFailed();
-                                }
-                                offLast = offCur + 1;
-                            }
-                            offCur++;
+                            if (fAttr)
+                                cchOutput += pfnOutput(pvArgOutput, "\"", 1);
+                            return cchOutput;
                         }
-                        if (offLast < offCur)
-                            cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
 
-                        while (--cchWidth >= cchStr)
-                            cchOutput += pfnOutput(pvArgOutput, " ", 1);
-                        if (fAttr)
-                            cchOutput += pfnOutput(pvArgOutput, "\"", 1);
-                        return cchOutput;
+                        default:
+                            AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
                     }
-
-                    default:
-                        AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
                 }
+                else if (chWhat == 'j')
+                {
+                    /* JSON string escaping. */
+                    char const chType = (*ppszFormat)[1];
+                    *ppszFormat += 2;
+                    switch (chType)
+                    {
+                        case 's':
+                        {
+                            const char *pszStr = va_arg(*pArgs, char *);
+                            size_t      cchOutput;
+                            ssize_t     cchStr;
+                            ssize_t     offCur;
+                            ssize_t     offLast;
+
+                            if (!VALID_PTR(pszStr))
+                                pszStr = "<NULL>";
+                            cchStr = RTStrNLen(pszStr, (unsigned)cchPrecision);
+
+                            cchOutput = pfnOutput(pvArgOutput, "\"", 1);
+                            if (!(fFlags & RTSTR_F_LEFT))
+                                while (--cchWidth >= cchStr)
+                                    cchOutput += pfnOutput(pvArgOutput, " ", 1);
+
+                            offLast = offCur = 0;
+                            while (offCur < cchStr)
+                            {
+                                unsigned int const uch = pszStr[offCur];
+                                if (   uch >= 0x5d
+                                    || (uch >= 0x20 && uch != 0x22 && uch != 0x5c))
+                                    offCur++;
+                                else
+                                {
+                                    if (offLast < offCur)
+                                        cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+                                    switch ((char)uch)
+                                    {
+                                        case '"':   cchOutput += pfnOutput(pvArgOutput, "\\\"", 2); break;
+                                        case '\\':  cchOutput += pfnOutput(pvArgOutput, "\\\\", 2); break;
+                                        case '/':   cchOutput += pfnOutput(pvArgOutput, "\\/", 2); break;
+                                        case '\b':  cchOutput += pfnOutput(pvArgOutput, "\\b", 2); break;
+                                        case '\f':  cchOutput += pfnOutput(pvArgOutput, "\\f", 2); break;
+                                        case '\n':  cchOutput += pfnOutput(pvArgOutput, "\\n", 2); break;
+                                        case '\t':  cchOutput += pfnOutput(pvArgOutput, "\\t", 2); break;
+                                        default:
+                                        {
+                                            RTUNICP     uc     = 0xfffd; /* replacement character */
+                                            const char *pszCur = &pszStr[offCur];
+                                            int rc = RTStrGetCpEx(&pszCur, &uc);
+                                            if (RT_SUCCESS(rc))
+                                                offCur += pszCur - &pszStr[offCur] - 1;
+                                            if (uc >= 0xfffe)
+                                                uc = 0xfffd;             /* replacement character */
+                                            szBuf[0] = '\\';
+                                            szBuf[1] = 'u';
+                                            szBuf[2] = g_szHexDigits[(uc >> 12) & 0xf];
+                                            szBuf[3] = g_szHexDigits[(uc >>  8) & 0xf];
+                                            szBuf[4] = g_szHexDigits[(uc >>  4) & 0xf];
+                                            szBuf[5] = g_szHexDigits[ uc        & 0xf];
+                                            szBuf[6] = '\0';
+                                            cchOutput += pfnOutput(pvArgOutput, szBuf, 6);
+                                            break;
+                                        }
+                                    }
+                                    offLast = ++offCur;
+                                }
+                            }
+                            if (offLast < offCur)
+                                cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+
+                            while (--cchWidth >= cchStr)
+                                cchOutput += pfnOutput(pvArgOutput, " ", 1);
+                            cchOutput += pfnOutput(pvArgOutput, "\"", 1);
+                            return cchOutput;
+                        }
+
+                        default:
+                            AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
+                    }
+                }
+                else if (chWhat == 'p')
+                {
+                    /* Percent encoded string (RTC-3986). */
+                    char const  chVariant = (*ppszFormat)[1];
+                    char const  chAddSafe = chVariant == 'p' ? '/'
+                                          : chVariant == 'q' ? '+' /* '+' in queries is problematic, so no escape. */
+                                          :                    '~' /* whatever */;
+                    size_t      cchOutput = 0;
+                    const char *pszStr    = va_arg(*pArgs, char *);
+                    ssize_t     cchStr;
+                    ssize_t     offCur;
+                    ssize_t     offLast;
+
+                    *ppszFormat += 2;
+                    AssertMsgBreak(chVariant == 'a' || chVariant == 'p' || chVariant == 'q' || chVariant == 'f',
+                                   ("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
+
+                    if (!VALID_PTR(pszStr))
+                        pszStr = "<NULL>";
+                    cchStr = RTStrNLen(pszStr, (unsigned)cchPrecision);
+
+                    if (!(fFlags & RTSTR_F_LEFT))
+                        while (--cchWidth >= cchStr)
+                            cchOutput += pfnOutput(pvArgOutput, "%20", 3);
+
+                    offLast = offCur = 0;
+                    while (offCur < cchStr)
+                    {
+                        ch = pszStr[offCur];
+                        if (   RT_C_IS_ALPHA(ch)
+                            || RT_C_IS_DIGIT(ch)
+                            || ch == '-'
+                            || ch == '.'
+                            || ch == '_'
+                            || ch == '~'
+                            || ch == chAddSafe)
+                            offCur++;
+                        else
+                        {
+                            if (offLast < offCur)
+                                cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+                            if (ch != ' ' || chVariant != 'f')
+                            {
+                                szBuf[0] = '%';
+                                szBuf[1] = g_szHexDigitsUpper[((uint8_t)ch >> 4) & 0xf];
+                                szBuf[2] = g_szHexDigitsUpper[(uint8_t)ch & 0xf];
+                                szBuf[3] = '\0';
+                                cchOutput += pfnOutput(pvArgOutput, szBuf, 3);
+                            }
+                            else
+                                cchOutput += pfnOutput(pvArgOutput, "+", 1);
+                            offLast = ++offCur;
+                        }
+                    }
+                    if (offLast < offCur)
+                        cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+
+                    while (--cchWidth >= cchStr)
+                        cchOutput += pfnOutput(pvArgOutput, "%20", 3);
+                }
+                else
+                    AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
                 break;
             }
-#endif /* IN_RING3 */
 
+#endif /* IN_RING3 */
 
             /*
              * Groups 6 - CPU Architecture Register Formatters.
