@@ -1,10 +1,10 @@
-/* $Id: errvars-generic.cpp 118412 2017-10-17 14:26:02Z bird $ */
+/* $Id: errvars-generic.cpp 127855 2019-01-01 01:45:53Z bird $ */
 /** @file
  * IPRT - Save and Restore Error Variables, generic stub implementation.
  */
 
 /*
- * Copyright (C) 2011-2017 Oracle Corporation
+ * Copyright (C) 2011-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -28,7 +28,7 @@
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
-#include <iprt/err.h>
+#include <iprt/errcore.h>
 #include "internal/iprt.h"
 
 #include <iprt/assert.h>
