@@ -1,10 +1,10 @@
-/* $Id: thread.h 136786 2020-04-01 12:40:26Z bird $ */
+/* $Id: thread.h 129750 2019-04-02 10:23:09Z bird $ */
 /** @file
  * IPRT - Internal RTThread header.
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2019 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,8 +24,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___thread_h
-#define ___thread_h
+#ifndef IPRT_INCLUDED_INTERNAL_thread_h
+#define IPRT_INCLUDED_INTERNAL_thread_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/types.h>
 #include <iprt/thread.h>
@@ -133,6 +136,9 @@ typedef RTTHREADINT *PRTTHREADINT;
 #define RTTHREADINT_FLAGS_MAIN       RT_BIT(3)
 /** @} */
 
+/** Counters for each thread type. */
+extern DECLHIDDEN(uint32_t volatile)   g_acRTThreadTypeStats[RTTHREADTYPE_END];
+
 
 /**
  * Initialize the native part of the thread management.
@@ -257,11 +263,19 @@ DECLHIDDEN(int)          rtThreadDoSetProcPriority(RTPROCPRIORITY enmPriority);
 DECLHIDDEN(void)         rtThreadClearTlsEntry(RTTLS iTls);
 DECLHIDDEN(void)         rtThreadTlsDestruction(PRTTHREADINT pThread); /* in tls-generic.cpp */
 #endif
-#ifdef RT_OS_WINDOWS
-DECLHIDDEN(void)         rtThreadWinTlsDestruction(void); /* in tls-win.cpp */
+
+/* thread-posix.cpp */
+#ifdef IN_RING3
+# if !defined(RT_OS_WINDOWS) && !defined(RT_OS_OS2) && !defined(RT_OS_DARWIN)
+#  define RTTHREAD_POSIX_WITH_CREATE_PRIORITY_PROXY
+# endif
+# ifdef RTTHREAD_POSIX_WITH_CREATE_PRIORITY_PROXY
+DECLHIDDEN(bool) rtThreadPosixPriorityProxyStart(void);
+DECLHIDDEN(int)  rtThreadPosixPriorityProxyCall(PRTTHREADINT pTargetThread, PFNRT pfnFunction, int cArgs, ...);
+# endif
 #endif
 
-#ifdef ___iprt_asm_h
+#ifdef IPRT_INCLUDED_asm_h
 
 /**
  * Gets the thread state.
@@ -290,4 +304,4 @@ DECLINLINE(void) rtThreadSetState(PRTTHREADINT pThread, RTTHREADSTATE enmNewStat
 
 RT_C_DECLS_END
 
-#endif
+#endif /* !IPRT_INCLUDED_INTERNAL_thread_h */
